import 'dart:io';

import 'package:android_path_provider/android_path_provider.dart';
import 'package:device_info_plus/device_info_plus.dart';
import 'package:flutter/foundation.dart';
import 'package:flutter/material.dart';
import 'package:flutter_downloader/flutter_downloader.dart';
import 'package:fluttertoast/fluttertoast.dart';
import 'package:global/global.dart';
import 'package:path_provider/path_provider.dart';
import 'package:permission_handler/permission_handler.dart';

class GUtils {
  //Utility function for getting headers used to access endpoints
  static Map<String, String> getBasicHeaders(
          {required String authorization, required String contentType}) =>
      <String, String>{
        'Authorization': authorization,
        'Content-type': contentType,
        'Accept': GValues.applnSlashJson
      };

  //Utility function for getting RoundBox decoration
  static BoxDecoration getRoundBoxDecoration({
    double radius = 6,
    double borderWidth = 0.75,
    Color fillColor = Colors.white,
    Color borderColor = const Color(0xFFD4D4D4),
    BorderRadiusGeometry? borderRadiusGeometry,
  }) {
    BorderSide borderSide = BorderSide(
      color: borderColor,
      width: borderWidth,
      style: borderWidth != 0 ? BorderStyle.solid : BorderStyle.none,
    );
    return BoxDecoration(
      color: fillColor,
      border: Border(
          top: borderSide,
          left: borderSide,
          bottom: borderSide,
          right: borderSide),
      borderRadius:
          borderRadiusGeometry ?? BorderRadius.all(Radius.circular(radius)),
    );
  }

  static OutlineInputBorder getRoundOutlineInputBorder({
    Color color = const Color(0xFFD4D4D4),
    double radius = 6,
    double width = 0.75,
  }) =>
      OutlineInputBorder(
        borderRadius: BorderRadius.all(Radius.circular(radius)),
        borderSide: BorderSide(
          color: color,
          width: width,
          style: width == 0 ? BorderStyle.none : BorderStyle.solid,
        ),
      );

  static RoundedRectangleBorder getRoundedRectBorder({
    double radius = 6,
    double width = 0.75,
    Color color = const Color(0xFFD4D4D4),
  }) =>
      RoundedRectangleBorder(
        side: BorderSide(
            width: width,
            color: color,
            style: width == 0 ? BorderStyle.none : BorderStyle.solid),
        borderRadius: BorderRadius.circular(radius),
      );

  static Text getText({
    required String title,
    bool? softWrap,
    FontWeight? fontWeight,
    Color color = const Color(0xFF4D4D4D),
    double fontSize = 13,
    TextOverflow? overflow,
    TextAlign textAlign = TextAlign.left,
    TextDecoration? decoration,
    Color? decorationColor,
    double? decorationThickness,
  }) =>
      Text(
        title,
        softWrap: softWrap,
        textAlign: textAlign,
        overflow: overflow,
        style: Global.textStyle.copyWith(
          fontWeight: fontWeight,
          color: color,
          decoration: decoration,
          decorationColor: decorationColor,
          fontSize: fontSize,
          decorationThickness: decorationThickness,
        ),
      );

  static Widget getFullScreenWidget({
    required BuildContext context,
    required Widget child,
    Widget? floatingActionButton,
  }) =>
      Scaffold(
        body: SafeArea(
          child: Container(
            height: getAvlblScreenHeight(context),
            color: Colors.white,
            child: child,
          ),
        ),
        floatingActionButton: floatingActionButton,
      );

  static Widget getActionElevatedButton({
    required String title,
    Color? backgroundColor,
    Color fontColor = Colors.white,
    double fontSize = 14.5,
    FontWeight fontWeight = FontWeight.w500,
    required Function() onPressed,
    double topBottomPadding = 10,
    int flex = 1,
  }) =>
      Row(
        mainAxisSize: MainAxisSize.min,
        children: [
          Expanded(
            flex: flex,
            child: ElevatedButton(
              onPressed: onPressed,
              style: getButtonStyle(
                  backgroundColor: backgroundColor ?? Global.themeColor,
                  topBottomPadding: topBottomPadding),
              child: Column(
                children: [
                  GUtils.getText(
                      title: title,
                      textAlign: TextAlign.center,
                      color: fontColor,
                      fontSize: fontSize,
                      fontWeight: fontWeight),
                ],
              ),
            ),
          ),
        ],
      );

  static ButtonStyle getButtonStyle({
    Color? backgroundColor,
    double topBottomPadding = 12,
  }) =>
      ButtonStyle(
          minimumSize: MaterialStateProperty.all(Size.zero),
          tapTargetSize: MaterialTapTargetSize.shrinkWrap,
          padding: MaterialStateProperty.all(
              EdgeInsets.fromLTRB(12, topBottomPadding, 12, topBottomPadding)),
          backgroundColor: MaterialStateColor.resolveWith(
              (states) => backgroundColor ?? Global.themeColor),
          overlayColor: MaterialStateColor.resolveWith(
              (states) => darken(backgroundColor ?? Global.themeColor, 0.1)),
          shape: backgroundColor == Colors.white
              ? MaterialStateProperty.all(getRoundedRectBorder())
              : null);

  //Utility function for giving a darkened color
  static Color darken(Color color, [double amount = .1]) {
    assert(amount >= 0 && amount <= 1);

    final hsl = HSLColor.fromColor(color);
    final hslDark = hsl.withLightness((hsl.lightness - amount).clamp(0.0, 1.0));

    return hslDark.toColor();
  }

  //Utility function for giving a lightened color
  static Color lighten(Color color, [double amount = .1]) {
    assert(amount >= 0 && amount <= 1);

    final hsl = HSLColor.fromColor(color);
    final hslLight =
        hsl.withLightness((hsl.lightness + amount).clamp(0.0, 1.0));

    return hslLight.toColor();
  }

  //Utility function for showing a toast
  static Future<bool?> showToast(
          {required String message,
          required Toast toastWithDuration,
          Color textColor = Colors.white,
          Color backgroundColor = const Color(0xFF838383),
          ToastGravity gravity = ToastGravity.BOTTOM}) =>
      Fluttertoast.showToast(
        msg: message,
        toastLength: toastWithDuration,
        gravity: gravity,
        fontSize: 14,
        textColor: textColor,
        backgroundColor: backgroundColor,
      );

  //Utility function for showing Progress Dialog
  static Future<dynamic> showProgressDialog({
    required GlobalKey<NavigatorState> navigatorKey,
  }) =>
      showMyDialog(
        navigatorKey: navigatorKey,
        dialogBody: getProgressDialogWidget(),
        canPop: false,
      );

  static Widget getProgressDialogWidget({bool isForNewRoute = true}) =>
      Material(
        type: MaterialType.transparency,
        child: SizedBox(
          width: double.infinity,
          child: Column(
            crossAxisAlignment: CrossAxisAlignment.center,
            mainAxisAlignment: MainAxisAlignment.center,
            mainAxisSize: MainAxisSize.max,
            children: <Widget>[
              CircularProgressIndicator(
                valueColor: AlwaysStoppedAnimation<Color>(
                    isForNewRoute ? Colors.white : const Color(0xFF4D4D4D)),
              )
            ],
          ),
        ),
      );

  static Future<T?> showMyDialog<T>({
    required Widget dialogBody,
    required GlobalKey<NavigatorState> navigatorKey,
    bool canPop = true,
    bool isAlertDialog = false,
  }) =>
      showDialog(
        context: navigatorKey.currentState!.context,
        builder: (dialogContext) => PopScope(
          canPop: canPop,
          child: StatefulBuilder(
            builder: (context, setState) => Material(
              type: MaterialType.transparency,
              child: Padding(
                  padding: MediaQuery.of(dialogContext).viewInsets,
                  child: Align(
                    alignment: Alignment.center,
                    child: Container(
                      padding: EdgeInsets.only(
                          left: isAlertDialog ? 40 : 20,
                          right: isAlertDialog ? 40 : 20),
                      child: dialogBody,
                    ),
                  )),
            ),
          ),
        ),
      );

  //Utility function for showing a custom alert dialog
  static Future<dynamic> showAlertDialog({
    required GlobalKey<NavigatorState> navigatorKey,
    required String title,
    required String content,
    Function? action,
    bool canPop = true,
  }) =>
      showMyDialog(
        navigatorKey: navigatorKey,
        dialogBody: GCustomAlertDialog(
          title: title,
          content: content,
          action: action,
        ),
        isAlertDialog: true,
        canPop: canPop,
      );

  //Utility function for getting available screen height in a device
  static double getAvlblScreenHeight(BuildContext context) =>
      MediaQuery.of(context).size.height -
      MediaQuery.of(context).padding.top -
      MediaQuery.of(context).padding.bottom;

  static printInDebug(Object object) {
    if (kDebugMode) {
      print(object);
    }
  }

  static void onFileDownloadStartRequest(
          {required GlobalKey<NavigatorState> navigatorKey,
          required String url}) =>
      /*Download Callback is implemented and registered in [main.dart] file*/
      showAlertDialog(
        navigatorKey: navigatorKey,
        title: GTexts.download,
        content: GTexts.continueWithFileDownload,
        action: () async {
          bool isStoragePermGranted;
          if (Platform.isIOS ||
              (await DeviceInfoPlugin().androidInfo).version.sdkInt > 28) {
            isStoragePermGranted = true;
          } else {
            switch (await Permission.storage.status) {
              case PermissionStatus.granted:
                isStoragePermGranted = true;
                break;

              default:
                /*We are handling only [PermissionStatus.granted] and [default] case,
               because 'Permission.storage.status' always returns either
               [PermissionStatus.granted] or [PermissionStatus.denied] status*/
                switch (await Permission.storage.request()) {
                  case PermissionStatus.granted:
                    isStoragePermGranted = true;
                    break;

                  case PermissionStatus.denied:
                    isStoragePermGranted = false;
                    break;

                  case PermissionStatus.permanentlyDenied:
                    isStoragePermGranted = false;
                    showAlertDialog(
                      navigatorKey: navigatorKey,
                      title: GTexts.storagePermission,
                      content: GTexts.storagePermNeeded,
                      action: () => openAppSettings().then((couldOpen) {
                        if (!couldOpen) {
                          showToast(
                              message: GTexts.appSettingsPageCouldNotBeOpened,
                              toastWithDuration: Toast.LENGTH_LONG);
                        }
                      }),
                    );
                    break;

                  default:
                    isStoragePermGranted = false;
                    break;
                }
                break;
            }
          }
          if (isStoragePermGranted) {
            String localStoragePath;
            if (Platform.isAndroid) {
              try {
                localStoragePath = await AndroidPathProvider.downloadsPath;
              } catch (_, __) {
                localStoragePath = (await getExternalStorageDirectory())!.path;
              }
            } else {
              localStoragePath =
                  (await getApplicationDocumentsDirectory()).absolute.path;
            }
            final savedDir = Directory(localStoragePath);
            if (!savedDir.existsSync()) {
              await savedDir.create();
            }
            FlutterDownloader.enqueue(
              url: url,
              saveInPublicStorage: true,
              savedDir: localStoragePath,
            );
          }
        },
      );

  static List<Shadow> getIconShadows() => [
        const Shadow(
            color: Colors.grey, blurRadius: 2.0, offset: Offset(0, 0.8))
      ];

  static String toCamelCase(String source) =>
      '${source[0].toUpperCase()}${source.substring(1).toLowerCase()}';
}
